@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2019-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -125,7 +125,7 @@ const toSegmentInfo = (aL, l, r) => ((r - l) << 24) | (aL + l);
 const roundToPageSize = v => (v + PAGE_SIZE-1) & ~(PAGE_SIZE-1);
 
 
-const BidiTrieContainer = class {
+class BidiTrieContainer {
 
     constructor(extraHandler) {
         const len = PAGE_SIZE * 4;
@@ -177,6 +177,19 @@ const BidiTrieContainer = class {
         this.lastStoredLen = this.lastStoredIndex = 0;
     }
 
+    createTrie() {
+        // grow buffer if needed
+        if ( (this.buf32[CHAR0_SLOT] - this.buf32[TRIE1_SLOT]) < CELL_BYTE_LENGTH ) {
+            this.growBuf(CELL_BYTE_LENGTH, 0);
+        }
+        const iroot = this.buf32[TRIE1_SLOT] >>> 2;
+        this.buf32[TRIE1_SLOT] += CELL_BYTE_LENGTH;
+        this.buf32[iroot+CELL_OR] = 0;
+        this.buf32[iroot+CELL_AND] = 0;
+        this.buf32[iroot+SEGMENT_INFO] = 0;
+        return iroot;
+    }
+
     matches(icell, ai) {
         const buf32 = this.buf32;
         const buf8 = this.buf8;
@@ -284,25 +297,9 @@ const BidiTrieContainer = class {
         return 1;
     }
 
-    createOne(args) {
-        if ( Array.isArray(args) ) {
-            return new this.STrieRef(this, args[0], args[1]);
-        }
-        // grow buffer if needed
-        if ( (this.buf32[CHAR0_SLOT] - this.buf32[TRIE1_SLOT]) < CELL_BYTE_LENGTH ) {
-            this.growBuf(CELL_BYTE_LENGTH, 0);
-        }
-        const iroot = this.buf32[TRIE1_SLOT] >>> 2;
-        this.buf32[TRIE1_SLOT] += CELL_BYTE_LENGTH;
-        this.buf32[iroot+CELL_OR] = 0;
-        this.buf32[iroot+CELL_AND] = 0;
-        this.buf32[iroot+SEGMENT_INFO] = 0;
-        return new this.STrieRef(this, iroot, 0);
-    }
-
-    compileOne(trieRef) {
-        return [ trieRef.iroot, trieRef.size ];
-    }
+    get $l() { return this.buf32[RESULT_L_SLOT] | 0; }
+    get $r() { return this.buf32[RESULT_R_SLOT] | 0; }
+    get $iu() { return this.buf32[RESULT_IU_SLOT] | 0; }
 
     add(iroot, aL0, n, pivot = 0) {
         const aR = n;
@@ -561,6 +558,14 @@ const BidiTrieContainer = class {
         }
     }
 
+    getExtra(iboundary) {
+        return this.buf32[iboundary+BCELL_EXTRA];
+    }
+
+    setExtra(iboundary, v) {
+        this.buf32[iboundary+BCELL_EXTRA] = v;
+    }
+
     optimize(shrink = false) {
         if ( shrink ) {
             this.shrinkBuf();
@@ -571,34 +576,19 @@ const BidiTrieContainer = class {
         };
     }
 
-    serialize(encoder) {
-        if ( encoder instanceof Object ) {
-            return encoder.encode(
-                this.buf32.buffer,
-                this.buf32[CHAR1_SLOT]
-            );
-        }
-        return Array.from(
-            new Uint32Array(
-                this.buf32.buffer,
-                0,
-                this.buf32[CHAR1_SLOT] + 3 >>> 2
-            )
+    toSelfie() {
+        return this.buf32.subarray(
+            0,
+            this.buf32[CHAR1_SLOT] + 3 >>> 2
         );
     }
 
-    unserialize(selfie, decoder) {
-        const shouldDecode = typeof selfie === 'string';
-        let byteLength = shouldDecode
-            ? decoder.decodeSize(selfie)
-            : selfie.length << 2;
+    fromSelfie(selfie) {
+        if ( selfie instanceof Uint32Array === false ) { return false; }
+        let byteLength = selfie.length << 2;
         if ( byteLength === 0 ) { return false; }
         this.reallocateBuf(byteLength);
-        if ( shouldDecode ) {
-            decoder.decode(selfie, this.buf8.buffer);
-        } else {
-            this.buf32.set(selfie);
-        }
+        this.buf32.set(selfie);
         return true;
     }
 
@@ -693,6 +683,80 @@ const BidiTrieContainer = class {
         return -1;
     }
 
+    dumpTrie(iroot) {
+        for ( const s of this.trieIterator(iroot) ) {
+            console.log(s);
+        }
+    }
+
+    trieIterator(iroot) {
+        return {
+            value: undefined,
+            done: false,
+            next() {
+                if ( this.icell === 0 ) {
+                    if ( this.forks.length === 0 ) {
+                        this.value = undefined;
+                        this.done = true;
+                        return this;
+                    }
+                    this.pattern = this.forks.pop();
+                    this.dir = this.forks.pop();
+                    this.icell = this.forks.pop();
+                }
+                const buf32 = this.container.buf32;
+                const buf8 = this.container.buf8;
+                for (;;) {
+                    const ialt = buf32[this.icell+CELL_OR];
+                    const v = buf32[this.icell+SEGMENT_INFO];
+                    const offset = v & 0x00FFFFFF;
+                    let i0 = buf32[CHAR0_SLOT] + offset;
+                    const len = v >>> 24;
+                    for ( let i = 0; i < len; i++ ) {
+                        this.charBuf[i] = buf8[i0+i];
+                    }
+                    if ( len !== 0 && ialt !== 0 ) {
+                        this.forks.push(ialt, this.dir, this.pattern);
+                    }
+                    const inext = buf32[this.icell+CELL_AND];
+                    if ( len !== 0 ) {
+                        const s = this.textDecoder.decode(
+                            new Uint8Array(this.charBuf.buffer, 0, len)
+                        );
+                        if ( this.dir > 0 ) {
+                            this.pattern += s;
+                        } else if ( this.dir < 0 ) {
+                            this.pattern = s + this.pattern;
+                        }
+                    }
+                    this.icell = inext;
+                    if ( len !== 0 ) { continue; }
+                    // boundary cell
+                    if ( ialt !== 0 ) {
+                        if ( inext === 0 ) {
+                            this.icell = ialt;
+                            this.dir = -1;
+                        } else {
+                            this.forks.push(ialt, -1, this.pattern);
+                        }
+                    }
+                    if ( offset !== 0 ) {
+                        this.value = { pattern: this.pattern, iextra: offset };
+                        return this;
+                    }
+                }
+            },
+            container: this,
+            icell: iroot,
+            charBuf: new Uint8Array(256),
+            pattern: '',
+            dir: 1,
+            forks: [],
+            textDecoder: new TextDecoder(),
+            [Symbol.iterator]() { return this; },
+        };
+    }
+
     async enableWASM(wasmModuleFetcher, path) {
         if ( typeof WebAssembly !== 'object' ) { return false; }
         if ( this.wasmMemory instanceof WebAssembly.Memory ) { return true; }
@@ -728,6 +792,13 @@ const BidiTrieContainer = class {
         return true;
     }
 
+    dumpInfo() {
+        return [
+            `Buffer size (Uint8Array): ${this.buf32[CHAR1_SLOT].toLocaleString('en')}`,
+            `WASM: ${this.wasmMemory === null ? 'disabled' : 'enabled'}`,
+        ].join('\n');
+    }
+
     //--------------------------------------------------------------------------
     // Private methods
     //--------------------------------------------------------------------------
@@ -816,103 +887,7 @@ const BidiTrieContainer = class {
             HAYSTACK_START + HAYSTACK_SIZE
         );
     }
-};
-
-/*******************************************************************************
-
-    Class to hold reference to a specific trie
-
-*/
-
-BidiTrieContainer.prototype.STrieRef = class {
-    constructor(container, iroot, size) {
-        this.container = container;
-        this.iroot = iroot;
-        this.size = size;
-    }
-
-    add(i, n, pivot = 0) {
-        const iboundary = this.container.add(this.iroot, i, n, pivot);
-        if ( iboundary !== 0 ) {
-            this.size += 1;
-        }
-        return iboundary;
-    }
-
-    getExtra(iboundary) {
-        return this.container.buf32[iboundary+BCELL_EXTRA];
-    }
-
-    setExtra(iboundary, v) {
-        this.container.buf32[iboundary+BCELL_EXTRA] = v;
-    }
-
-    matches(i) {
-        return this.container.matches(this.iroot, i);
-    }
-
-    dump() {
-        for ( const s of this ) {
-            console.log(s);
-        }
-    }
-
-    get $l() { return this.container.buf32[RESULT_L_SLOT] | 0; }
-    get $r() { return this.container.buf32[RESULT_R_SLOT] | 0; }
-    get $iu() { return this.container.buf32[RESULT_IU_SLOT] | 0; }
-
-    [Symbol.iterator]() {
-        return {
-            value: undefined,
-            done: false,
-            next: function() {
-                if ( this.icell === 0 ) {
-                    if ( this.forks.length === 0 ) {
-                        this.value = undefined;
-                        this.done = true;
-                        return this;
-                    }
-                    this.charPtr = this.forks.pop();
-                    this.icell = this.forks.pop();
-                }
-                for (;;) {
-                    const idown = this.container.buf32[this.icell+CELL_OR];
-                    if ( idown !== 0 ) {
-                        this.forks.push(idown, this.charPtr);
-                    }
-                    const v = this.container.buf32[this.icell+SEGMENT_INFO];
-                    let i0 = this.container.buf32[CHAR0_SLOT] + (v & 0x00FFFFFF);
-                    const i1 = i0 + (v >>> 24);
-                    while ( i0 < i1 ) {
-                        this.charBuf[this.charPtr] = this.container.buf8[i0];
-                        this.charPtr += 1;
-                        i0 += 1;
-                    }
-                    this.icell = this.container.buf32[this.icell+CELL_AND];
-                    if ( this.icell === 0 ) {
-                        return this.toPattern();
-                    }
-                    if ( this.container.buf32[this.icell+SEGMENT_INFO] === 0 ) {
-                        this.icell = this.container.buf32[this.icell+CELL_AND];
-                        return this.toPattern();
-                    }
-                }
-            },
-            toPattern: function() {
-                this.value = this.textDecoder.decode(
-                    new Uint8Array(this.charBuf.buffer, 0, this.charPtr)
-                );
-                return this;
-            },
-            container: this.container,
-            icell: this.iroot,
-            charBuf: new Uint8Array(256),
-            charPtr: 0,
-            forks: [],
-            textDecoder: new TextDecoder()
-        };
-    }
-};
+}
 
 /******************************************************************************/
 
@@ -954,4 +929,4 @@ const getWasmModule = (( ) => {
 
 /******************************************************************************/
 
-export { BidiTrieContainer };
+export default BidiTrieContainer;
